/*
 * Copyright 2008-2009 SpringSource
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.springsource.bundlor.support.propertysubstitution;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.junit.Before;
import org.junit.Test;
import org.springframework.util.StringUtils;

import com.springsource.bundlor.util.BundleManifestUtils;
import com.springsource.bundlor.util.SimpleManifestContents;
import com.springsource.bundlor.util.SimpleParserLogger;
import com.springsource.util.osgi.manifest.BundleManifest;
import com.springsource.util.osgi.manifest.parse.HeaderDeclaration;
import com.springsource.util.osgi.manifest.parse.HeaderParser;
import com.springsource.util.osgi.manifest.parse.HeaderParserFactory;
import com.springsource.util.parser.manifest.ManifestContents;

public class PlaceholderManifestTemplateModifierTests {

    private PlaceholderManifestAndTemplateModifier modifier;

    @Before
    public void createModifier() {
        Properties properties = new Properties();
        properties.put("com.springsource.version", "1.2.3.qualifer");
        modifier = new PlaceholderManifestAndTemplateModifier(properties);
    }

    @Test
    public void basicVersionExpansion() {
        ManifestContents manifestTemplate = new SimpleManifestContents();
        manifestTemplate.getMainAttributes().put("Import-Template", "com.foo;version=\"${com.springsource.version:[=.=,+1)}\"");

        modifier.modify(manifestTemplate);
        assertImportTemplate("com.foo", "[1.2,2)", manifestTemplate);
    }

    @Test
    public void patternVersionExpansion() {
        ManifestContents manifestTemplate = new SimpleManifestContents();
        manifestTemplate.getMainAttributes().put("Import-Template", "com.foo;version=\"${com.springsource.version:sample}\"");
        manifestTemplate.getMainAttributes().put("Version-Patterns", "sample;pattern=\"[=.=,+1)\"");

        modifier.modify(manifestTemplate);
        assertImportTemplate("com.foo", "[1.2,2)", manifestTemplate);
    }

    @Test
    public void testPlaceholderReplacement() {
        System.getProperties().put("domain.name", "org.springframework");
        PlaceholderManifestAndTemplateModifier modifier = new PlaceholderManifestAndTemplateModifier(System.getProperties());

        ManifestContents manifestTemplate = new SimpleManifestContents();
        manifestTemplate.getMainAttributes().put("Bundle-SymbolicName", "${domain.name}.core.${user.name}");
        manifestTemplate.getMainAttributes().put("Bundle-Version", "2.5.1");

        modifier.modify(manifestTemplate);
        BundleManifest manifest = BundleManifestUtils.createBundleManifest(manifestTemplate);
        assertEquals("org.springframework.core." + System.getProperty("user.name"), manifest.getBundleSymbolicName().getSymbolicName());
    }

    private void assertImportTemplate(String packageName, String version, ManifestContents manifestTemplate) {
        BundleManifest manifest = BundleManifestUtils.createBundleManifest(manifestTemplate);
        List<HeaderDeclaration> headers = parseTemplate(manifest.getHeader("Import-Template"));
        boolean found = false;
        for (HeaderDeclaration header : headers) {
            if (header.getNames().get(0).equals(packageName) && header.getAttributes().get("version").equals(version)) {
                found = true;
                break;
            }
        }

        assertTrue("Did not find Import-Template entry for " + packageName + " - " + version, found);
    }

    private List<HeaderDeclaration> parseTemplate(String template) {
        if (StringUtils.hasText(template)) {
            HeaderParser parser = HeaderParserFactory.newHeaderParser(new SimpleParserLogger());
            return parser.parseHeader(template);
        }
        return new ArrayList<HeaderDeclaration>(0);
    }
}
